@@ -8,7 +8,7 @@ jQuery(function($){
   // ***************************************************************
   //  Plush defaults
 
-  refreshRate:          $.cookie('plushRefreshRate')     ? $.cookie('plushRefreshRate')  : 30, // refresh rate in seconds
+  refreshRate:          $.cookie('plushRefreshRate')     ? $.cookie('plushRefreshRate')  : 4, // refresh rate in seconds
   containerWidth:       $.cookie('plushContainerWidth')  ? $.cookie('plushContainerWidth')  : '100%', // width of all elements on page
   queuePerPage:         $.cookie('plushQueuePerPage')    ? $.cookie('plushQueuePerPage') : 5, // pagination - nzbs per page
   histPerPage:          $.cookie('plushHistPerPage')     ? $.cookie('plushHistPerPage')  : 5, // pagination - nzbs per page
@@ -16,7 +16,7 @@ jQuery(function($){
   confirmDeleteHistory: $.cookie('plushConfirmDeleteHistory') == 0 ? false : true, // confirm history nzb removal
   blockRefresh:         $.cookie('plushBlockRefresh') == 0 ? false : true, // prevent refreshing when hovering queue
   failedOnly:           $.cookie('plushFailedOnly') == 1 ? 1 : 0, // prevent refreshing when hovering queue
-  multiOps:             $.cookie('plushMultiOps') == 1 ? true : false, // is multi-operations menu visible in queue
+  multiOps:             $.cookie('plushMultiOps') == 0 ? false : true, // is multi-operations menu visible in queue
   noTopMenu:            $.cookie('plushNoTopMenu') == 1 ? false : true, // is top menu visible
   multiOpsChecks:       null,
 
@@ -40,6 +40,7 @@ jQuery(function($){
     $('#addID').click(function(){ // also works when hitting enter because of <form>
       if ($('#addID_input').val()!='URL') {
         $.ajax({
+          headers: {"Cache-Control": "no-cache"},
           type: "POST",
           url: "tapi",
           data: {
@@ -118,15 +119,19 @@ jQuery(function($){
 
   // fix for touch devices -- toggle visibility
   $('.sprite_q_menu_pausefor').bind('touchend', function(e) {
-      e.preventDefault();
-      if( $(this).hasClass('sprite_q_menu_pauseforsfHover') ) {
-        $(this).find("ul").toggle();
+      if (! $.browser.safari) {
+        e.preventDefault();
+        if( $(this).hasClass('sprite_q_menu_pauseforsfHover') ) {
+          $(this).find("ul").toggle();
+        }
       }
   });
   $('.sprite_q_queue').bind('touchend', function(e) {
-      e.preventDefault();
-      if( $(this).hasClass('sprite_q_queuesfHover') ) {
-        $(this).find("ul").toggle();
+      if (! $.browser.safari) {
+        e.preventDefault();
+        if( $(this).hasClass('sprite_q_queuesfHover') ) {
+          $(this).find("ul").toggle();
+        }
       }
   });
 
@@ -154,6 +159,7 @@ jQuery(function($){
     else
       $('#speed-wrapper .sprite_q_menu_pausefor').removeClass('sprite_q_menu_pausefor_on');
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'config', name:'set_speedlimit', value: str, apikey: $.plush.apikey}
@@ -209,6 +215,7 @@ jQuery(function($){
     else
       $('.sprite_q_queue').removeClass('sprite_q_queue_on');
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:'change_complete_action', value: $(this).val(), apikey: $.plush.apikey}
@@ -230,6 +237,7 @@ jQuery(function($){
       value="all";
     }
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:'delete', value:value, del_files:del_files, apikey: $.plush.apikey},
@@ -254,6 +262,7 @@ jQuery(function($){
       case 'sortSizeDesc':  sort='size';    dir='desc'; break;
     }
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:'sort', sort: sort, dir: dir, apikey: $.plush.apikey},
@@ -268,6 +277,7 @@ jQuery(function($){
       minutes = prompt($(event.target).attr('title'));
     $.plush.SetQueuePauseInfo(true,minutes+':00');
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'config', name:'set_pause', value: minutes, apikey: $.plush.apikey},
@@ -278,6 +288,7 @@ jQuery(function($){
   // Get Bookmarks
   $('#get_bookmarks_now').click(function() {
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'newzbin', name:'get_bookmarks', apikey: $.plush.apikey},
@@ -288,6 +299,7 @@ jQuery(function($){
   // Reset Quota
   $('#reset_quota_now').click(function() {
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'reset_quota', apikey: $.plush.apikey},
@@ -298,6 +310,7 @@ jQuery(function($){
   // Get RSS
   $('#get_rss_now').click(function() {
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'rss_now', apikey: $.plush.apikey},
@@ -308,6 +321,7 @@ jQuery(function($){
   // Get Watched folder
   $('#get_watched_now').click(function() {
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'watched_now', apikey: $.plush.apikey},
@@ -471,6 +485,7 @@ jQuery(function($){
       $('#pause_resume').removeClass('sprite_q_pause_on').addClass('sprite_q_pause');
       $('#pause_int').html("");
       $.ajax({
+        headers: {"Cache-Control": "no-cache"},
         type: "POST",
         url: "tapi",
         data: {mode:'resume', apikey: $.plush.apikey}
@@ -479,6 +494,7 @@ jQuery(function($){
       $('#pause_resume').removeClass('sprite_q_pause').addClass('sprite_q_pause_on');
       $('#pause_int').html("");
       $.ajax({
+        headers: {"Cache-Control": "no-cache"},
         type: "POST",
         url: "tapi",
         data: {mode:'pause', apikey: $.plush.apikey}
@@ -522,6 +538,7 @@ jQuery(function($){
     if ($(this).hasClass('sprite_ql_grip_resume_on')) {
       $(this).toggleClass('sprite_ql_grip_resume_on').toggleClass('sprite_ql_grip_pause_on');
       $.ajax({
+        headers: {"Cache-Control": "no-cache"},
         type: "POST",
         url: "tapi",
         data: {mode:'queue', name:'pause', value: pid, apikey: $.plush.apikey}
@@ -529,6 +546,7 @@ jQuery(function($){
     } else {
       $(this).toggleClass('sprite_ql_grip_resume_on').toggleClass('sprite_ql_grip_pause_on');
       $.ajax({
+        headers: {"Cache-Control": "no-cache"},
         type: "POST",
         url: "tapi",
         data: {mode:'queue', name:'resume', value: pid, apikey: $.plush.apikey}
@@ -573,6 +591,7 @@ jQuery(function($){
     var nzbid = $(this).parent().parent().attr('id');
     var oldPos = $('#'+nzbid)[0].rowIndex + $.plush.queuecurpage * $.plush.queuePerPage;
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:'priority', value: nzbid, value2: $(this).val(), apikey: $.plush.apikey},
@@ -595,6 +614,7 @@ jQuery(function($){
     var val = $(this).parent().parent().attr('id');
     var cval = $(this).attr('class').split(" ")[0]; // ignore added "hovering" class
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode: cval, value: val, value2: $(this).val(), apikey: $.plush.apikey},
@@ -683,6 +703,7 @@ $.plush.queueprevslots = $.plush.queuenoofslots; // for the next refresh
     if (table.tBodies[0].rows[i].id == row.id) {
       val2 = (i + $.plush.queuecurpage * $.plush.queuePerPage);
       $.ajax({
+        headers: {"Cache-Control": "no-cache"},
         type: "POST",
         url: "tapi",
         data: {mode:'switch', value: row.id, value2: val2, apikey: $.plush.apikey},
@@ -768,6 +789,7 @@ $("a","#multiops_inputs").click(function(e){
 
   if ($('#multi_status').val())
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:$('#multi_status').val(), value: nzo_ids, apikey: $.plush.apikey}
@@ -775,6 +797,7 @@ $("a","#multiops_inputs").click(function(e){
 
   if ($('#multi_cat').val())
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode: 'change_cat', value: nzo_ids, value2: $('#multi_cat').val(), apikey: $.plush.apikey}
@@ -782,6 +805,7 @@ $("a","#multiops_inputs").click(function(e){
 
   if ($('#multi_priority').val())
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode:'queue', name:'priority', value: nzo_ids, value2: $('#multi_priority').val(), apikey: $.plush.apikey}
@@ -789,6 +813,7 @@ $("a","#multiops_inputs").click(function(e){
 
   if ($('#multi_pp').val())
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode: 'change_opts', value: nzo_ids, value2: $('#multi_pp').val(), apikey: $.plush.apikey}
@@ -796,6 +821,7 @@ $("a","#multiops_inputs").click(function(e){
 
   if ($('#multi_script').val())
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
       data: {mode: 'change_script', value: nzo_ids, value2: $('#multi_script').val(), apikey: $.plush.apikey}
@@ -866,9 +892,10 @@ $("a","#multiops_inputs").click(function(e){
       value="failed";
     }
     $.ajax({
+      headers: {"Cache-Control": "no-cache"},
       type: "POST",
       url: "tapi",
-      data: {mode:'history', name:'delete', value:value, del_files:del_files, apikey: $.plush.apikey},
+      data: {mode:'history', name:'delete', value:value, del_files:del_files, search: $('#historySearchBox').val(), apikey: $.plush.apikey},
       success: function(){
         $.colorbox.close();
         $.plush.modalOpen=false;
@@ -955,6 +982,7 @@ $("a","#multiops_inputs").click(function(e){
   $.plush.pendingHistoryRefresh = true;
   $.colorbox.close();
   $.ajax({
+    headers: {"Cache-Control": "no-cache"},
     type: "POST",
     url: "tapi",
     data: {mode:mode, name:'delete', value: delid, del_files: del_files, apikey: $.plush.apikey},
@@ -1081,6 +1109,7 @@ $.plush.histprevslots = $.plush.histnoofslots; // for the next refresh
 
   // Fetch updated content from queue.tmpl
   $.ajax({
+    headers: {"Cache-Control": "no-cache"},
     type: "POST",
     url: "queue/",
     data: {start: ( page * $.plush.queuePerPage ), limit: $.plush.queuePerPage},
@@ -1136,6 +1165,7 @@ $.plush.histprevslots = $.plush.histnoofslots; // for the next refresh
 
 
   $.ajax({
+    headers: {"Cache-Control": "no-cache"},
     type: "POST",
     url: "history/",
     data: data,
@@ -1212,12 +1242,16 @@ $.plush.histprevslots = $.plush.histnoofslots; // for the next refresh
   SetQueueETAStats : function(speed,kbpersec,timeleft,eta) {
 
   // ETA/speed stats at top of queue
-  if (kbpersec < 1 && $.plush.paused)
+  if (kbpersec < 100 && $.plush.paused) {
     $('#stats_eta').html('&mdash;');
-  else
+    $('#stats_speed').html('&mdash;');
+    $('#time-left').attr('title','&mdash;');  // Tooltip on "time left"
+  }
+  else {
     $('#stats_eta').html(timeleft);
-  $('#stats_speed').html(speed+"B/s");
-  $('#time-left').attr('title',eta);  // Tooltip on "time left"
+    $('#stats_speed').html(speed+"B/s");
+    $('#time-left').attr('title',eta);  // Tooltip on "time left"
+  }
 },
 
 
