@@ -52,7 +52,7 @@ from sabnzbd.utils.json import JsonWriter
 from sabnzbd.utils.pathbrowser import folders_at_path
 from sabnzbd.misc import loadavg, to_units, diskfree, disktotal, get_ext, \
                          get_filename, int_conv, globber, time_format, remove_all, \
-                         starts_with_path
+                         starts_with_path, cat_convert
 from sabnzbd.encoding import xml_name, unicoder, special_fixer, platform_encode, html_escape
 from sabnzbd.postproc import PostProcessor
 from sabnzbd.articlecache import ArticleCache
@@ -293,9 +293,11 @@ def _api_addfile(name, output, kwargs):
         #Side effect of next line is that attribute .value is created
         #which is needed to make add_nzbfile() work
         size = name.length
-    else:
+    elif hasattr(name, 'value'):
         size = len(name.value)
-    if name is not None and name.filename and size:
+    else:
+        size = 0
+    if name is not None and size and name.filename:
         cat = kwargs.get('cat')
         xcat = kwargs.get('xcat')
         if not cat and xcat:
@@ -344,7 +346,7 @@ def _api_addlocalfile(name, output, kwargs):
 
                 if get_ext(name) in ('.zip', '.rar'):
                     res = sabnzbd.dirscanner.ProcessArchiveFile(\
-                        fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True)
+                        fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True, nzbname=nzbname)
                 elif get_ext(name) in ('.nzb', '.gz'):
                     res = sabnzbd.dirscanner.ProcessSingleFile(\
                         fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True, nzbname=nzbname)
@@ -422,6 +424,8 @@ def _api_history(name, output, kwargs):
     limit = kwargs.get('limit')
     search = kwargs.get('search')
     failed_only = kwargs.get('failed_only')
+    if not limit:
+        limit = cfg.history_limit()
 
     if name == 'delete':
         special = value.lower()
@@ -430,10 +434,10 @@ def _api_history(name, output, kwargs):
             history_db = cherrypy.thread_data.history_db
             if special in ('all', 'failed'):
                 if del_files:
-                    del_job_files(history_db.get_failed_paths())
-                history_db.remove_failed()
+                    del_job_files(history_db.get_failed_paths(search))
+                history_db.remove_failed(search)
             if special in ('all', 'completed'):
-                history_db.remove_completed()
+                history_db.remove_completed(search)
             return report(output)
         elif value:
             jobs = value.split(',')
@@ -461,33 +465,6 @@ def _api_get_files(name, output, kwargs):
     else:
         return report(output, _MSG_NO_VALUE)
 
-def _api_addurl(names, output, kwargs):
-    """ API: accepts name, output, pp, script, cat, priority, nzbname """
-    pp = kwargs.get('pp')
-    script = kwargs.get('script')
-    cat = kwargs.get('cat')
-    priority = kwargs.get('priority')
-    nzbnames = kwargs.get('nzbname')
-    if not isinstance(names, list):
-        names = [names]
-    if not isinstance(nzbnames, list):
-        nzbnames = [nzbnames]
-
-    for n in xrange(len(names)):
-        name = names[n]
-        if n < len(nzbnames):
-            nzbname = nzbnames[n]
-        else:
-            nzbname = ''
-        if name:
-            name = name.strip()
-        sabnzbd.add_url(name, pp, script, cat, priority, nzbname)
-
-    if len(names) > 0:
-        return report(output)
-    else:
-        return report(output, _MSG_NO_VALUE)
-
 
 _RE_NEWZBIN_URL = re.compile(r'/browse/post/(\d+)')
 def _api_addid(names, output, kwargs):
@@ -581,10 +558,13 @@ def _api_auth(name, output, kwargs):
     if not cfg.disable_key():
         auth = 'badkey'
         key = kwargs.get('key', '')
-        if key == cfg.nzb_key():
-            auth = 'nzbkey'
-        if key == cfg.api_key():
+        if not key:
             auth = 'apikey'
+        else:
+            if key == cfg.nzb_key():
+                auth = 'nzbkey'
+            if key == cfg.api_key():
+                auth = 'apikey'
     elif cfg.username() and cfg.password():
         auth = 'login'
     return report(output, keyword='auth', data=auth)
@@ -670,7 +650,7 @@ def _api_test_email(name, output, kwargs):
     pack['unpack'] = ['action 1', 'action 2']
     res = sabnzbd.emailer.endjob('I had a d\xe8ja vu', 123, 'unknown', True,
                                  os.path.normpath(os.path.join(cfg.complete_dir.get_path(), '/unknown/I had a d\xe8ja vu')),
-                                 123*MEBI, pack, 'my_script', 'Line 1\nLine 2\nLine 3\nd\xe8ja vu\n', 0)
+                                 123*MEBI, None, pack, 'my_script', 'Line 1\nLine 2\nLine 3\nd\xe8ja vu\n', 0)
     if res == 'Email succeeded':
         res = None
     return report(output, error=res)
@@ -796,7 +776,7 @@ _api_table = {
     'fullstatus'      : _api_fullstatus,
     'history'         : _api_history,
     'get_files'       : _api_get_files,
-    'addurl'          : _api_addurl,
+    'addurl'          : _api_addid,
     'addid'           : _api_addid,
     'pause'           : _api_pause,
     'resume'          : _api_resume,
@@ -1127,7 +1107,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
             slot['mbdone_fmt'] = locale.format('%d', int(mb-mbleft), True)
         slot['size'] = format_bytes(bytes)
         slot['sizeleft'] = format_bytes(bytesleft)
-        if not Downloader.do.paused and status != 'Paused' and status != 'Fetching' and not found_active:
+        if not Downloader.do.paused and status not in (Status.PAUSED, Status.FETCHING) and not found_active:
             if status == Status.CHECKING:
                 slot['status'] = Status.CHECKING
             else:
@@ -1151,7 +1131,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
             slot['percentage'] = "%s" % (int(((mb-mbleft) / mb) * 100))
         slot['missing'] = missing
 
-        if status in (Status.PAUSED, Status.CHECKING):
+        if Downloader.do.paused or Downloader.do.postproc or status not in (Status.DOWNLOADING, Status.QUEUED):
             slot['timeleft'] = '0:00:00'
             slot['eta'] = 'unknown'
         else:
@@ -1165,7 +1145,11 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
                 datestart = datetime.datetime.now()
                 slot['eta'] = 'unknown'
 
-        slot['avg_age'] = calc_age(average_date, bool(trans))
+        if status == Status.GRABBING:
+            slot['avg_age'] = '---'
+        else:
+            slot['avg_age'] = calc_age(average_date, bool(trans))
+
         slot['verbosity'] = ""
         if web_dir:
             finished = []
@@ -1252,6 +1236,16 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
     return info, pnfo_list, bytespersec, verbose_list, dictn
 
 
+#------------------------------------------------------------------------------
+def fast_queue():
+    """ Return paused, bytes_left, bpsnow, time_left """
+    bytes_left = NzbQueue.do.remaining()
+    paused = Downloader.do.paused
+    bpsnow = BPSMeter.do.get_bps()
+    time_left = calc_timeleft(bytes_left, bpsnow)
+    return paused, bytes_left, bpsnow, time_left
+
+
 #------------------------------------------------------------------------------
 def qstatus_data():
     """Build up the queue status as a nested object and output as a JSON object
@@ -1279,16 +1273,17 @@ def qstatus_data():
 
     state = "IDLE"
     if Downloader.do.paused:
-        state = "PAUSED"
+        state = Status.PAUSED
     elif qnfo[QNFO_BYTES_LEFT_FIELD] / MEBI > 0:
-        state = "DOWNLOADING"
+        state = Status.DOWNLOADING
 
     status = {
         "state" : state,
+        "pp_active" : not PostProcessor.do.empty(),
         "paused" : Downloader.do.paused,
         "pause_int" : scheduler.pause_int(),
-        "kbpersec" : BPSMeter.do.get_bps() / KIBI,
-        "speed" : to_units(BPSMeter.do.get_bps(), dec_limit=1),
+        "kbpersec" : bpsnow / KIBI,
+        "speed" : to_units(bpsnow, dec_limit=1),
         "mbleft" : qnfo[QNFO_BYTES_LEFT_FIELD] / MEBI,
         "mb" : qnfo[QNFO_BYTES_FIELD] / MEBI,
         "noofslots" : len(pnfo_list),
@@ -1549,7 +1544,8 @@ def build_header(prim, webdir=''):
     if not color:
         color = ''
 
-    header = { 'T': Ttemplate, 'Tspec': Tspec, 'Tx' : Ttemplate, 'version':sabnzbd.__version__, 'paused': Downloader.do.paused,
+    header = { 'T': Ttemplate, 'Tspec': Tspec, 'Tx' : Ttemplate, 'version':sabnzbd.__version__,
+               'paused': Downloader.do.paused or Downloader.do.postproc,
                'pause_int': scheduler.pause_int(), 'paused_all': sabnzbd.PAUSED_ALL,
                'uptime':uptime, 'color_scheme':color }
     speed_limit = Downloader.do.get_limit()
@@ -1600,13 +1596,13 @@ def build_header(prim, webdir=''):
     header['left_quota'] = to_units(BPSMeter.do.left)
 
     status = ''
-    if Downloader.do.paused:
+    if Downloader.do.paused or Downloader.do.postproc:
         status = Status.PAUSED
     elif bytespersec > 0:
         status = Status.DOWNLOADING
     else:
         status = 'Idle'
-    header['status'] = "%s" % status
+    header['status'] = status
 
     anfo  = ArticleCache.do.cache_info()
 
@@ -1686,9 +1682,11 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     # Aquire the db instance
     try:
         history_db = cherrypy.thread_data.history_db
+        close_db = False
     except:
         # Required for repairs at startup because Cherrypy isn't active yet
         history_db = get_history_handle()
+        close_db = True
 
     # Fetch history items
     if not h_limit:
@@ -1754,6 +1752,9 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     total_items += full_queue_size
     fetched_items = len(items)
 
+    if close_db:
+        history_db.close()
+
     return (items, fetched_items, total_items)
 
 
@@ -1945,3 +1946,13 @@ def del_from_section(kwargs):
     else:
         return False
 
+
+#------------------------------------------------------------------------------
+def history_remove_failed():
+    """ Remove all failed jobs from history, including files """
+    logging.info('Scheduled removal of all failed jobs')
+    history_db = get_history_handle()
+    del_job_files(history_db.get_failed_paths())
+    history_db.remove_failed()
+    history_db.close()
+    del history_db
