@@ -31,10 +31,12 @@ import sabnzbd
 from sabnzbd.encoding import TRANS, UNTRANS, unicode2local, name_fixer, \
      reliable_unpack_names, unicoder, latin1, platform_encode
 from sabnzbd.utils.rarfile import RarFile, is_rarfile
-from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv
+from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv, \
+                         flag_file, real_path, globber
 from sabnzbd.tvsort import SeriesSorter
 import sabnzbd.cfg as cfg
-from constants import Status
+from sabnzbd.constants import Status, QCHECK_FILE, RENAMES_FILE
+load_data = save_data = None
 
 if sabnzbd.WIN32:
     try:
@@ -77,6 +79,7 @@ CURL_COMMAND = None
 def find_programs(curdir):
     """Find external programs
     """
+    global load_data, save_data
     def check(path, program):
         p = os.path.abspath(os.path.join(path, program))
         if os.access(p, os.X_OK):
@@ -84,6 +87,10 @@ def find_programs(curdir):
         else:
             return None
 
+    # Another crazy Python import bug work-around
+    load_data = sabnzbd.load_data
+    save_data = sabnzbd.save_data
+
     if sabnzbd.DARWIN:
         try:
             os_version = subprocess.Popen("sw_vers -productVersion", stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True).stdout.read()
@@ -124,7 +131,7 @@ def find_programs(curdir):
 def external_processing(extern_proc, complete_dir, filename, msgid, nicename, cat, group, status):
     """ Run a user postproc script, return console output and exit value
     """
-    command = [str(extern_proc), str(complete_dir), str(filename), \
+    command = [str(extern_proc), str(complete_dir), str(filename),
                str(nicename), str(msgid), str(cat), str(group), str(status)]
 
     if extern_proc.endswith('.py') and (sabnzbd.WIN32 or not os.access(extern_proc, os.X_OK)):
@@ -132,7 +139,7 @@ def external_processing(extern_proc, complete_dir, filename, msgid, nicename, ca
     stup, need_shell, command, creationflags = build_command(command)
     env = fix_env()
 
-    logging.info('Running external script %s(%s, %s, %s, %s, %s, %s, %s)', \
+    logging.info('Running external script %s(%s, %s, %s, %s, %s, %s, %s)',
                  extern_proc, complete_dir, filename, nicename, msgid, cat, group, status)
 
     try:
@@ -257,8 +264,8 @@ def match_ts(file):
 
 
 def clean_up_joinables(names):
-    ''' Remove joinable files and their .1 backups
-    '''
+    """ Remove joinable files and their .1 backups
+    """
     for name in names:
         if os.path.exists(name):
             logging.debug("Deleting %s", name)
@@ -275,8 +282,8 @@ def clean_up_joinables(names):
                 pass
 
 def get_seq_number(name):
-    ''' Return sequence number if name as an int
-    '''
+    """ Return sequence number if name as an int
+    """
     head, tail = os.path.splitext(name)
     if tail == '.ts':
         match, set, num = match_ts(name)
@@ -296,7 +303,8 @@ def file_join(nzo, workdir, workdir_complete, delete, joinables):
 
     # Create matching sets from the list of files
     joinable_sets = {}
-    set = match = num = None
+    joinable_set = None
+    set = num = None
     for joinable in joinables:
         head, tail = os.path.splitext(joinable)
         if tail == '.ts':
@@ -353,7 +361,7 @@ def file_join(nzo, workdir, workdir_complete, delete, joinables):
             # Finish up
             joined_file.flush()
             joined_file.close()
-            newfiles.append(joinable_set)
+            newfiles.append(filename)
 
             if seq_error:
                 msg = T('Incomplete sequence of joinable files')
@@ -383,6 +391,7 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
         When 'one_folder' is set, all files will be in a single folder
     """
     extracted_files = []
+    success = False
 
     rar_sets = {}
     for rar in rars:
@@ -437,7 +446,7 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
                 except OSError:
                     logging.warning(Ta('Deleting %s failed!'), latin1(rar))
 
-                brokenrar = '%s.1' % (rar)
+                brokenrar = '%s.1' % rar
 
                 if os.path.exists(brokenrar):
                     logging.info("Deleting %s", brokenrar)
@@ -456,17 +465,21 @@ def rar_extract(rarfile, numrars, one_folder, nzo, setname, extraction_path):
     """
 
     fail = 0
+    new_files = None
+    rars = []
     if nzo.password:
         passwords = [nzo.password]
     else:
         passwords = []
+        # Append meta passwords, to prevent changing the original list
+        passwords.extend(nzo.meta.get('password', []))
         pw_file = cfg.password_file.get_path()
         if pw_file:
             try:
                 pwf = open(pw_file, 'r')
-                passwords = pwf.read().split('\n')
+                lines = pwf.read().split('\n')
                 # Remove empty lines and space-only passwords and remove surrounding spaces
-                passwords = [pw.strip('\r\n ') for pw in passwords if pw.strip('\r\n ')]
+                passwords.extend([pw.strip('\r\n ') for pw in lines if pw.strip('\r\n ')])
                 pwf.close()
                 logging.info('Read the passwords file %s', pw_file)
             except IOError:
@@ -523,17 +536,23 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
         action = 'e'
     else:
         action = 'x'
+    if cfg.overwrite_files():
+        overwrite = '-o+' # Enable overwrite
+        rename = '-o+'    # Dummy
+    else:
+        overwrite = '-o-' # Disable overwrite
+        rename = '-or'    # Auto renaming
     if sabnzbd.WIN32:
         # Use all flags
-        command = ['%s' % RAR_COMMAND, action, '-idp', '-o-', '-or', '-ai', password,
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, '-ai', password,
                    '%s' % rarfile, '%s/' % extraction_path]
     elif RAR_PROBLEM:
         # Use only oldest options (specifically no "-or")
-        command = ['%s' % RAR_COMMAND, action, '-idp', '-o-', password,
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, password,
                    '%s' % rarfile, '%s/' % extraction_path]
     else:
         # Don't use "-ai" (not needed for non-Windows)
-        command = ['%s' % RAR_COMMAND, action, '-idp', '-o-', '-or', password,
+        command = ['%s' % RAR_COMMAND, action, '-idp', overwrite, rename, password,
                    '%s' % rarfile, '%s/' % extraction_path]
 
     if cfg.ignore_unrar_dates():
@@ -550,7 +569,7 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
     if p.stdin:
         p.stdin.close()
 
-    nzo.set_action_line(T('Unpacking'), '00/%02d' % (numrars))
+    nzo.set_action_line(T('Unpacking'), '00/%02d' % numrars)
 
     # Loop over the output from rar!
     curr = 0
@@ -592,7 +611,20 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
             nzo.fail_msg = T('Unpacking failed, write error or disk is full?')
             msg = ('[%s] ' + Ta('Unpacking failed, write error or disk is full?')) % setname
             nzo.set_unpack_info('Unpack', unicoder(msg), set=setname)
-            logging.warning(Ta('ERROR: write error (%s)'), line[11:])
+            logging.error(Ta('ERROR: write error (%s)'), line[11:])
+            fail = 1
+
+        elif line.startswith('Cannot create'):
+            line2 = proc.readline()
+            if 'must not exceed 260' in line2:
+                nzo.fail_msg = T('Unpacking failed, path is too long')
+                msg = '[%s] %s: %s' % (Ta('Unpacking failed, path is too long'), setname, line[13:])
+                logging.error(Ta('ERROR: path too long (%s)'), line[13:])
+            else:
+                nzo.fail_msg = T('Unpacking failed, write error or disk is full?')
+                msg = '[%s] %s: %s' % (Ta('Unpacking failed, write error or disk is full?'), setname, line[13:])
+                logging.error(Ta('ERROR: write error (%s)'), line[13:])
+            nzo.set_unpack_info('Unpack', unicoder(msg), set=setname)
             fail = 1
 
         elif line.startswith('ERROR: '):
@@ -620,14 +652,14 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
         else:
             m = re.search(r'^(Extracting|Creating|...)\s+(.*?)\s+OK\s*$', line)
             if m:
-                extracted.append(TRANS(m.group(2)))
+                extracted.append(real_path(extraction_path, TRANS(m.group(2))))
 
         if fail:
             if proc:
                 proc.close()
             p.wait()
 
-            return (fail, (), ())
+            return fail, (), ()
 
     if proc:
         proc.close()
@@ -669,7 +701,7 @@ def rar_extract_core(rarfile, numrars, one_folder, nzo, setname, extraction_path
     nzo.set_unpack_info('Unpack', '[%s] %s' % (unicoder(setname), msg), set=setname)
     logging.info('%s', msg)
 
-    return (0, extracted, rarfiles)
+    return 0, extracted, rarfiles
 
 #------------------------------------------------------------------------------
 # (Un)Zip Functions
@@ -714,7 +746,7 @@ def unzip(nzo, workdir, workdir_complete, delete, one_folder, zips):
                 except OSError:
                     logging.warning(Ta('Deleting %s failed!'), latin1(_zip))
 
-                brokenzip = '%s.1' % (_zip)
+                brokenzip = '%s.1' % _zip
 
                 if os.path.exists(brokenzip):
                     logging.info("Deleting %s", brokenzip)
@@ -764,8 +796,9 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
     parfile = os.path.join(workdir, parfile_nzf.filename)
 
     old_dir_content = os.listdir(workdir)
-    used_joinables = []
-    joinables = []
+    used_joinables = ()
+    joinables = ()
+    used_par2 = ()
     setpars = pars_of_set(workdir, setname)
     result = readd = False
 
@@ -779,6 +812,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
         result = True
 
     if not result:
+        flag_file(workdir, QCHECK_FILE, True)
         nzo.status = Status.REPAIRING
         result = False
         readd = False
@@ -788,8 +822,8 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
 
             joinables, zips, rars, ts = build_filelists(workdir, None, check_rar=False)
 
-            finished, readd, pars, datafiles, used_joinables = PAR_Verify(parfile, parfile_nzf, nzo,
-                                                                          setname, joinables)
+            finished, readd, pars, datafiles, used_joinables, used_par2 = PAR_Verify(parfile, parfile_nzf, nzo,
+                                                                                     setname, joinables)
 
             if finished:
                 result = True
@@ -858,6 +892,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
                 if f in setpars:
                     deletables.append(os.path.join(workdir, f))
             deletables.extend(used_joinables)
+            deletables.extend(used_par2)
             for filepath in deletables:
                 if filepath in joinables:
                     joinables.remove(filepath)
@@ -876,6 +911,10 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
 
 
 _RE_BLOCK_FOUND = re.compile('File: "([^"]+)" - found \d+ of \d+ data blocks from "([^"]+)"')
+_RE_IS_MATCH_FOR = re.compile('File: "([^"]+)" - is a match for "([^"]+)"')
+_RE_LOADING_PAR2 = re.compile('Loading "([^"]+)"\.')
+_RE_LOADED_PAR2 = re.compile('Loaded (\d+) new packets')
+
 def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
     """ Run par2 on par-set """
     if cfg.never_repair():
@@ -884,6 +923,8 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
         cmd = 'r'
     retry_classic = False
     used_joinables = []
+    used_par2 = []
+    extra_par2_name = None
     #set the current nzo status to "Verifying...". Used in History
     nzo.status = Status.VERIFYING
     start = time()
@@ -892,7 +933,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
     logging.debug('Par2-classic = %s', classic)
 
     import sabnzbd.assembler
-    if (sabnzbd.assembler.GetMD5Hashes(parfile)[1] and not classic) or not PAR2C_COMMAND:
+    if (sabnzbd.assembler.GetMD5Hashes(parfile, True)[1] and not classic) or not PAR2C_COMMAND:
         if cfg.par_option():
             command = [str(PAR2_COMMAND), cmd, str(cfg.par_option().strip()), parfile]
         else:
@@ -906,9 +947,17 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
         if setname in joinable:
             command.append(joinable)
 
+    # Append the wildcard for this set
+    wildcard = '%s*' % os.path.join(os.path.split(parfile)[0], setname)
+    if len(globber(wildcard, None)) < 2:
+        # Support bizarre naming conventions
+        wildcard = os.path.join(os.path.split(parfile)[0], '*')
+    command.append(wildcard)
+
     stup, need_shell, command, creationflags = build_command(command)
     logging.debug('Starting par2: %s', command)
 
+    lines = []
     try:
         p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
@@ -922,6 +971,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
         # Set up our variables
         pars = []
         datafiles = []
+        renames = {}
 
         linebuf = ''
         finished = 0
@@ -949,7 +999,19 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
             if line == '':
                 continue
 
-            # And off we go
+            if 'Repairing:' not in line:
+                lines.append(line)
+
+            if extra_par2_name and line.startswith('Loading:') and line.endswith('%'):
+                continue
+            if extra_par2_name and line.startswith('Loaded '):
+                m = _RE_LOADED_PAR2.search(line)
+                if m and int(m.group(1)) > 0:
+                    used_par2.append(os.path.join(nzo.downpath, extra_par2_name))
+                extra_par2_name = None
+                continue
+            extra_par2_name = None
+
             if line.startswith('Invalid option specified'):
                 msg = T('[%s] PAR2 received incorrect options, check your Config->Switches settings') % unicoder(setname)
                 nzo.set_unpack_info('Repair', msg, set=setname)
@@ -970,8 +1032,14 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 start = time()
                 verified = 1
 
-            elif line.startswith('Main packet not found'):
-                ## Initialparfile probaly didn't decode properly,
+            elif line.startswith('Loading "'):
+                # Found an extra par2 file. Only the next line will tell whether it's usable
+                m = _RE_LOADING_PAR2.search(line)
+                if m and m.group(1).lower().endswith('.par2'):
+                    extra_par2_name = m.group(1)
+
+            elif line.startswith('Main packet not found') or 'The recovery file does not exist' in line:
+                ## Initialparfile probably didn't decode properly,
                 logging.info(Ta('Main packet not found...'))
 
                 extrapars = parfile_nzf.extrapars
@@ -981,14 +1049,21 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 ## Look for the smallest par2file
                 block_table = {}
                 for nzf in extrapars:
-                    block_table[int_conv(nzf.blocks)] = nzf
+                    if not nzf.completed:
+                        block_table[int_conv(nzf.blocks)] = nzf
 
                 if block_table:
                     nzf = block_table[min(block_table.keys())]
 
                     logging.info("Found new par2file %s", nzf.filename)
 
+                    ## Move from extrapar list to files to be downloaded
                     nzo.add_parfile(nzf)
+                    extrapars.remove(nzf)
+                    ## Now set new par2 file as primary par2
+                    nzo.partable[setname] = nzf
+                    nzf.extrapars= extrapars
+                    parfile_nzf = []
                     ## mark for readd
                     readd = True
                 else:
@@ -1050,7 +1125,9 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                         extrapar_list = block_table[block_size]
 
                         if extrapar_list:
-                            nzo.add_parfile(extrapar_list.pop())
+                            new_nzf = extrapar_list.pop()
+                            nzo.add_parfile(new_nzf)
+                            if new_nzf in extrapars: extrapars.remove(new_nzf)
                             added_blocks += block_size
 
                         else:
@@ -1079,7 +1156,7 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
             elif line.startswith('Repairing:'):
                 chunks = line.split()
                 per = float(chunks[-1][:-1])
-                nzo.set_action_line(T('Repairing'), '%2d%%' % (per))
+                nzo.set_action_line(T('Repairing'), '%2d%%' % per)
                 nzo.status = Status.REPAIRING
 
             elif line.startswith('Repair complete'):
@@ -1105,6 +1182,33 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
                 # Hit a bug in par2-tbb, retry with par2-classic
                 retry_classic = True
 
+            elif ' cannot be renamed to ' in line:
+                if not classic and sabnzbd.WIN32:
+                    # Hit a bug in par2-tbb, retry with par2-classic
+                    retry_classic = True
+                else:
+                    msg = unicoder(line.strip())
+                    nzo.fail_msg = msg
+                    msg = u'[%s] %s' % (unicoder(setname), msg)
+                    nzo.set_unpack_info('Repair', msg, set=setname)
+                    nzo.status = Status.FAILED
+
+            # File: "oldname.rar" - is a match for "newname.rar".
+            elif 'is a match for' in line:
+                m = _RE_IS_MATCH_FOR.search(line)
+                if m:
+                    old_name = m.group(1)
+                    new_name = m.group(2)
+                    logging.debug('PAR2 will rename "%s" to "%s"', old_name, new_name)
+                    renames[new_name] = old_name
+
+            elif 'No details available for recoverable file' in line:
+                msg = unicoder(line.strip())
+                nzo.fail_msg = msg
+                msg = u'[%s] %s' % (unicoder(setname), msg)
+                nzo.set_unpack_info('Repair', msg, set=setname)
+                nzo.status = Status.FAILED
+
             elif not verified:
                 if line.startswith('Verifying source files'):
                     nzo.set_action_line(T('Verifying'), '01/%02d' % verifytotal)
@@ -1143,11 +1247,20 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
         else:
             raise WindowsError(err)
 
+    logging.debug('PAR2 output was\n%s', '\n'.join(lines))
+
+    # If successful, add renamed files to the collection
+    if finished and renames:
+        previous = load_data(RENAMES_FILE, nzo.workpath, remove=False)
+        for name in previous or {}:
+            renames[name] = previous[name]
+        save_data(renames, RENAMES_FILE, nzo.workpath)
+
     if retry_classic:
         logging.debug('Retry PAR2-joining with par2-classic')
         return PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=True)
     else:
-        return (finished, readd, pars, datafiles, used_joinables)
+        return finished, readd, pars, datafiles, used_joinables, used_par2
 
 #-------------------------------------------------------------------------------
 
@@ -1198,7 +1311,7 @@ def build_command(command):
         if need_shell:
             command = list2cmdline(command)
 
-    return (stup, need_shell, command, creationflags)
+    return stup, need_shell, command, creationflags
 
 # Sort the various RAR filename formats properly :\
 def rar_sort(a, b):
@@ -1254,7 +1367,10 @@ def build_filelists(workdir, workdir_complete, check_rar=True):
 
     zips = [f for f in filelist if ZIP_RE.search(f)]
 
-    rars = [f for f in filelist if RAR_RE.search(f) and is_rarfile(f)]
+    if check_rar:
+        rars = [f for f in filelist if RAR_RE.search(f) and is_rarfile(f)]
+    else:
+        rars = [f for f in filelist if RAR_RE.search(f)]
 
     ts = [f for f in filelist if TS_RE.search(f) and f not in joinables]
 
@@ -1263,7 +1379,7 @@ def build_filelists(workdir, workdir_complete, check_rar=True):
     logging.debug("build_filelists(): rars: %s", rars)
     logging.debug("build_filelists(): ts: %s", ts)
 
-    return (joinables, zips, rars, ts)
+    return joinables, zips, rars, ts
 
 
 def QuickCheck(set, nzo):
@@ -1277,13 +1393,12 @@ def QuickCheck(set, nzo):
     nzf_list = nzo.finished_files
 
     for file in md5pack:
-        file = name_fixer(file)
         if sabnzbd.misc.on_cleanup_list(file, False):
             result = True
             continue
         found = False
         for nzf in nzf_list:
-            if file == name_fixer(nzf.filename):
+            if file == nzf.filename:
                 found = True
                 if (nzf.md5sum is not None) and nzf.md5sum == md5pack[file]:
                     logging.debug('Quick-check of file %s OK', file)
@@ -1334,32 +1449,35 @@ def unrar_check(rar):
 def sfv_check(sfv_path):
     """ Verify files using SFV file,
         input: full path of sfv, file are assumed to be relative to sfv
-        returns: True when all files verified OK
+        returns: List of failing files or [] when all is OK
     """
+    failed = []
     try:
         fp = open(sfv_path, 'r')
     except:
         logging.info('Cannot open SFV file %s', sfv_path)
-        return False
+        failed.append(unicoder(sfv_path))
+        return failed
     root = os.path.split(sfv_path)[0]
-    status = True
     for line in fp:
         line = line.strip('\n\r ')
-        if line[0] != ';':
+        if line and line[0] != ';':
             x = line.rfind(' ')
-            filename = line[:x].strip()
-            checksum = line[x:].strip()
-            path = os.path.join(root, platform_encode(filename))
-            if os.path.exists(path):
-                if crc_check(path, checksum):
-                    logging.debug('File %s passed SFV check', path)
+            if x > 0:
+                filename = platform_encode(line[:x].strip())
+                checksum = line[x:].strip()
+                path = os.path.join(root, filename)
+                if os.path.exists(path):
+                    if crc_check(path, checksum):
+                        logging.debug('File %s passed SFV check', path)
+                    else:
+                        logging.info('File %s did not pass SFV check', latin1(path))
+                        failed.append(unicoder(filename))
                 else:
-                    logging.warning('File %s did not pass SFV check', latin1(path))
-                    status = False
-            else:
-                logging.warning('File %s mssing in SFV check', latin1(path))
+                    logging.info('File %s missing in SFV check', latin1(path))
+                    failed.append(unicoder(filename))
     fp.close()
-    return status
+    return failed
 
 
 def crc_check(path, target_crc):
@@ -1384,6 +1502,7 @@ def crc_check(path, target_crc):
 def analyse_show(name):
     """ Do a quick SeasonSort check and return basic facts """
     job = SeriesSorter(name, None, None)
+    job.match(force=True)
     if job.is_match():
         job.get_values()
     info = job.show_info
@@ -1455,7 +1574,10 @@ def list2cmdline(lst):
 #------------------------------------------------------------------------------
 # Work-around for the failure of Python2.5 on Windows to support IPV6 with HTTPS
 
-def get_from_url(url):
+def get_from_url(url, timeout=None):
+    """ Retrieve URL and return content
+        `timeout` sets non-standard timeout and skips when on Windows
+    """
     if 'https:' in url and sabnzbd.WIN32 and sys.version_info < (2,6) and sabnzbd.newsunpack.CURL_COMMAND:
         command = [sabnzbd.newsunpack.CURL_COMMAND, "-k", url]
         stup, need_shell, command, creationflags = build_command(command)
@@ -1467,6 +1589,14 @@ def get_from_url(url):
         p.wait()
     else:
         import urllib2
-        s = urllib2.urlopen(url)
-        output = s.read()
+        if sys.version_info < (2, 6):
+            timeout = 0
+        try:
+            if timeout:
+                s = urllib2.urlopen(url, timeout=timeout)
+            else:
+                s = urllib2.urlopen(url)
+            output = s.read()
+        except:
+            output = None
     return output
