@@ -37,7 +37,7 @@ import sabnzbd
 from sabnzbd.constants import sample_match, GIGI, ATTRIB_FILE, JOB_ADMIN, \
                               DEFAULT_PRIORITY, LOW_PRIORITY, NORMAL_PRIORITY, \
                               HIGH_PRIORITY, PAUSED_PRIORITY, TOP_PRIORITY, DUP_PRIORITY, \
-                              Status
+                              RENAMES_FILE, Status
 from sabnzbd.misc import to_units, cat_to_opts, cat_convert, sanitize_foldername, \
                          get_unique_path, get_admin_path, remove_all, format_source_url, \
                          sanitize_filename, globber, sanitize_foldername, int_conv, \
@@ -54,7 +54,8 @@ RE_NORMAL  = re.compile(r"(.+)(\.nzb)", re.I)
 SUBJECT_FN_MATCHER = re.compile(r'"([^"]*)"')
 RE_SAMPLE = re.compile(sample_match, re.I)
 PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)\+(\d*)\.par2', re.I)
-
+REJECT_PAR2_RE = re.compile(r'\.par2\.\d+', re.I) # Reject duplicate par2 files
+RE_NORMAL_NAME = re.compile(r'\.\w{2,5}$') # Test reasonably sized extension at the end
 
 ################################################################################
 # Article                                                                      #
@@ -154,6 +155,7 @@ NzbFileMapper = (
     ('import_finished',              'import_finished'),
     ('md5sum',                       'md5sum'),
     ('valid',                        'valid'),
+    ('completed',                    'completed')
 )
 
 
@@ -169,9 +171,7 @@ class NzbFile(TryList):
         self.filename = None
         self.type = None
 
-        match = re.search(SUBJECT_FN_MATCHER, subject)
-        if match:
-            self.filename = match.group(1).strip('"')
+        self.filename = name_extractor(subject)
 
         self.is_par2 = False
         self.vol = None
@@ -191,6 +191,7 @@ class NzbFile(TryList):
         self.nzo = nzo
         self.nzf_id = sabnzbd.get_new_id("nzf", nzo.workpath)
         self.deleted = False
+        self.completed = False
 
         self.valid = False
         self.import_finished = False
@@ -272,7 +273,7 @@ class NzbFile(TryList):
     @property
     def completed(self):
         """ Is this file completed? """
-        return not bool(self.articles)
+        return self.import_finished and not bool(self.articles)
 
     @property
     def lowest_partnum(self):
@@ -321,6 +322,11 @@ class NzbParser(xml.sax.handler.ContentHandler):
         self.in_group = False
         self.in_segments = False
         self.in_segment = False
+        self.in_head = False
+        self.in_meta = False
+        self.meta_type = ''
+        self.meta_types = {}
+        self.meta_content = []
         self.filename = ''
         self.avg_age = 0
         self.valids = 0
@@ -348,12 +354,8 @@ class NzbParser(xml.sax.handler.ContentHandler):
             self.in_segments = True
 
         elif name == 'file' and self.in_nzb:
-            subject = attrs.get('subject', '')
-            match = re.search(SUBJECT_FN_MATCHER, subject)
-            if match:
-                self.filename = match.group(1).strip('"').strip()
-            else:
-                self.filename = subject.strip()
+            subject = attrs.get('subject', '').strip()
+            self.filename = subject
 
             if self.filter and RE_SAMPLE.search(subject):
                 logging.info('Skipping sample file %s', subject)
@@ -377,6 +379,16 @@ class NzbParser(xml.sax.handler.ContentHandler):
         elif name == 'groups' and self.in_nzb and self.in_file:
             self.in_groups = True
 
+        elif name == 'head' and self.in_nzb:
+            self.in_head = True
+
+        elif name == 'meta' and self.in_nzb and self.in_head:
+            self.in_meta = True
+            meta_type = attrs.get('type')
+            if meta_type:
+                self.meta_type = meta_type.lower()
+            self.meta_content = []
+
         elif name == 'nzb':
             self.in_nzb = True
 
@@ -385,6 +397,8 @@ class NzbParser(xml.sax.handler.ContentHandler):
             self.group_name.append(content)
         elif self.in_segment:
             self.article_id.append(content)
+        elif self.in_meta:
+            self.meta_content.append(content)
 
     def endElement(self, name):
         if name == 'group' and self.in_group:
@@ -439,12 +453,24 @@ class NzbParser(xml.sax.handler.ContentHandler):
                     sabnzbd.remove_data(nzf.nzf_id, self.nzo.workpath)
                 self.skipped_files += 1
 
+        elif name == 'head':
+            self.in_head = False
+
+        elif name == 'meta':
+            self.in_meta = False
+            if self.meta_type:
+                if self.meta_type not in self.meta_types:
+                    self.meta_types[self.meta_type] = []
+                self.meta_types[self.meta_type].append(''.join(self.meta_content))
+
         elif name == 'nzb':
             self.in_nzb = False
 
     def endDocument(self):
         """ End of the file """
         self.nzo.groups = self.groups
+        self.nzo.meta = self.meta_types
+        logging.debug('META-DATA = %s', self.nzo.meta)
         files = max(1, self.valids)
         self.nzo.avg_stamp = self.avg_age / files
         self.nzo.avg_date = datetime.datetime.fromtimestamp(self.avg_age / files)
@@ -504,7 +530,9 @@ NzbObjectMapper = (
     ('oversized',                    'oversized'),     # Was detected as oversized
     ('create_group_folder',          'create_group_folder'),
     ('precheck',                     'precheck'),
-    ('incomplete',                   'incomplete')     # Was detected as incomplete
+    ('incomplete',                   'incomplete'),    # Was detected as incomplete
+    ('reuse',                        'reuse'),
+    ('meta',                         'meta')           # Meta-date from 1.1 type NZB
 )
 
 class NzbObject(TryList):
@@ -540,6 +568,7 @@ class NzbObject(TryList):
         self.work_name = work_name
         self.final_name = work_name
 
+        self.meta = {}
         self.created = False        # dirprefixes + work_name created
         self.bytes = 0              # Original bytesize
         self.bytes_downloaded = 0   # Downloaded byte
@@ -591,6 +620,7 @@ class NzbObject(TryList):
         self.oversized = False
         self.precheck = False
         self.incomplete = False
+        self.reuse = reuse
         if self.status == Status.QUEUED and not reuse:
             self.precheck = cfg.pre_check()
             if self.precheck:
@@ -698,12 +728,23 @@ class NzbObject(TryList):
 
         if not self.files and not reuse:
             self.purge_data(keep_basic=False)
-            if self.url:
-                logging.warning(Ta('Empty NZB file %s') + ' [%s]', filename, self.url)
+            if cfg.warn_empty_nzb():
+                mylog = logging.warning
             else:
-                logging.warning(Ta('Empty NZB file %s'), filename)
+                mylog = logging.info
+            if self.url:
+                mylog(Ta('Empty NZB file %s') + ' [%s]', filename, self.url)
+            else:
+                mylog(Ta('Empty NZB file %s'), filename)
             raise ValueError
 
+        if cat is None:
+            for metacat in self.meta.get('category', ()):
+                metacat = cat_convert(metacat)
+                if metacat:
+                    cat = metacat
+                    break
+
         if cat is None:
             for grp in self.groups:
                 cat = cat_convert(grp)
@@ -810,6 +851,7 @@ class NzbObject(TryList):
         if nzf in self.files:
             self.files.remove(nzf)
             self.finished_files.append(nzf)
+            nzf.import_finished = True
             nzf.deleted = True
         return not bool(self.files)
 
@@ -818,38 +860,38 @@ class NzbObject(TryList):
             nzf.reset_all_try_lists()
         self.reset_try_list()
 
-    def postpone_pars(self, nzf, head):
-        """ Move all vol-par files matching 'head' to the extrapars table """
-        self.partable[head] = nzf
-        self.extrapars[head] = []
-        nzf.extrapars = self.extrapars[head]
+    def postpone_pars(self, nzf, parset):
+        """ Move all vol-par files matching 'parset' to the extrapars table """
+        self.partable[parset] = nzf
+        self.extrapars[parset] = []
+        nzf.extrapars = self.extrapars[parset]
+        lparset = parset.lower()
         for xnzf in self.files[:]:
-            name = xnzf.filename
+            name = xnzf.filename or platform_encode(xnzf.subject)
             # Move only when not current NZF and filename was extractable from subject
             if name and nzf is not xnzf:
-                name = name.lower()
-                if head.lower() in name and '.vol' in name and name.endswith('.par2'):
-                    self.extrapars[head].append(xnzf)
-                    self.files.remove(xnzf)
+                head, vol, block = analyse_par2(name)
+                if head and matcher(lparset, head.lower()):
+                    xnzf.set_par2(parset, vol, block)
+                    self.extrapars[parset].append(xnzf)
+                    if not self.precheck:
+                        self.files.remove(xnzf)
 
     def handle_par2(self, nzf, file_done):
-        ## Special treatment for first part of par2 file
+        """ Check if file is a par2 and build up par2 collection
+        """
         fn = nzf.filename
         if fn:
+            # We have a real filename now
             fn = fn.strip()
             lfn = fn.lower()
-            if (not nzf.is_par2) and fn and lfn.endswith('.par2'):
-                par2match = PROBABLY_PAR2_RE.search(fn)
-                if par2match:
-                    head = par2match.group(1)
-                    vol = par2match.group(2)
-                    block = par2match.group(3)
-                elif lfn.endswith('.par2'):
-                    head = os.path.splitext(fn)[0]
-                    vol = block = 0
-                    par2match = True
+            if not nzf.is_par2:
+                head, vol, block = analyse_par2(fn)
                 ## Is a par2file and repair mode activated
-                if par2match and (self.repair or cfg.allow_streaming()):
+                if head and (self.repair or cfg.allow_streaming()):
+                    ## Skip if mini-par2 is not complete
+                    if not block and nzf.bytes_left:
+                        return
                     nzf.set_par2(head, vol, block)
                     ## Already got a parfile for this set?
                     if head in self.partable:
@@ -867,15 +909,17 @@ class NzbObject(TryList):
                         ## This file either has more blocks,
                         ## or initialparfile is already decoded
                         else:
-                            if not file_done:
-                                nzf.reset_try_list()
+                            if file_done:
                                 if nzf in self.files: self.files.remove(nzf)
-                                self.extrapars[head].append(nzf)
+                                if nzf not in self.extrapars[head]: self.extrapars[head].append(nzf)
+                            else:
+                                nzf.reset_try_list()
+
                     ## No par2file in this set yet, set this as
                     ## initialparfile
                     else:
                         self.postpone_pars(nzf, head)
-                ## Is not a par2file or nothing todo
+                ## Is not a par2file or nothing to do
                 else:
                     pass
         ## No filename in seg 1? Probably not uu or yenc encoded
@@ -890,11 +934,20 @@ class NzbObject(TryList):
 
         if file_done:
             self.remove_nzf(nzf)
+            if not self.reuse and not self.precheck and cfg.fail_hopeless() and not self.check_quality(99)[0]:
+                #set the nzo status to return "Queued"
+                self.status = Status.QUEUED
+                self.set_download_report()
+                self.fail_msg = T('Aborted, cannot be completed')
+                self.set_unpack_info('Download', self.fail_msg, unique=False)
+                logging.debug('Abort job "%s", due to impossibility to complete it', self.final_name_pw_clean)
+                return True, True, True
 
         if reset:
             self.reset_try_list()
 
-        self.handle_par2(nzf, file_done)
+        if file_done:
+            self.handle_par2(nzf, file_done)
 
         post_done = False
         if not self.files:
@@ -911,6 +964,15 @@ class NzbObject(TryList):
         """
         # Get a list of already present files
         files = [os.path.basename(f) for f in globber(wdir) if os.path.isfile(f)]
+
+        # Substitute renamed files
+        renames = sabnzbd.load_data(RENAMES_FILE, self.workpath, remove=True)
+        if renames:
+            for name in renames:
+                if name in files:
+                    files.remove(name)
+                    files.append(renames[name])
+
         # Looking for the longest name first, minimizes the chance on a mismatch
         files.sort(lambda x, y: len(y) - len(x))
 
@@ -924,6 +986,8 @@ class NzbObject(TryList):
                 subject = sanitize_filename(latin1(nzf.subject))
                 if (nzf.filename == filename) or (subject == filename) or (filename in subject):
                     nzf.filename = filename
+                    nzf.completed = True
+                    nzf.bytes_left = 0
                     self.handle_par2(nzf, file_done=True)
                     self.remove_nzf(nzf)
                     nzfs.remove(nzf)
@@ -940,6 +1004,8 @@ class NzbObject(TryList):
                 self.files_table[nzf.nzf_id] = nzf
                 self.bytes += nzf.bytes
                 nzf.filename = filename
+                nzf.completed = True
+                nzf.bytes_left = 0
                 self.handle_par2(nzf, file_done=True)
                 self.remove_nzf(nzf)
                 logging.info('File %s added to job', filename)
@@ -1007,17 +1073,16 @@ class NzbObject(TryList):
         self.incomplete = False
 
     def add_parfile(self, parfile):
-        self.files.append(parfile)
-        if parfile.extrapars:
+        if parfile not in self.files:
+            self.files.append(parfile)
+        if parfile.extrapars and parfile in parfile.extrapars:
             parfile.extrapars.remove(parfile)
-        else:
-            logging.debug('PARFILE without EXTRAPARS %s', parfile.filename or parfile.subject)
 
     def remove_parset(self, setname):
         self.partable.pop(setname)
 
     __re_quick_par2_check = re.compile('\.par2\W*', re.I)
-    def check_quality(self):
+    def check_quality(self, req_ratio=0):
         """ Determine amount of articles present on servers
             and return (gross available, nett) bytes
         """
@@ -1028,7 +1093,8 @@ class NzbObject(TryList):
         for nzf_id in self.files_table:
             nzf = self.files_table[nzf_id]
             assert isinstance(nzf, NzbFile)
-            short += nzf.bytes_left
+            if nzf.deleted:
+                short += nzf.bytes_left
             if self.__re_quick_par2_check.search(nzf.subject):
                 pars += nzf.bytes
                 anypars = True
@@ -1037,7 +1103,7 @@ class NzbObject(TryList):
         have = need + pars - short
         ratio = float(have) / float(max(1, need))
         if anypars:
-            enough = ratio * 100.0 >= float(cfg.req_completion_rate())
+            enough = ratio * 100.0 >= (req_ratio or float(cfg.req_completion_rate()))
         else:
             enough = have >= need
         logging.debug('Download Quality: enough=%s, have=%s, need=%s, ratio=%s', enough, have, need, ratio)
@@ -1061,15 +1127,18 @@ class NzbObject(TryList):
             msg1 = T('Downloaded in %s at an average of %sB/s') % (complete_time, to_units(avg_bps*1024, dec_limit=1))
             bad = self.nzo_info.get('bad_art_log', [])
             miss = self.nzo_info.get('missing_art_log', [])
+            killed = self.nzo_info.get('killed_art_log', [])
             dups = self.nzo_info.get('dup_art_log', [])
-            msg2 = msg3 = msg4 = ''
+            msg2 = msg3 = msg4 = msg5 = ''
             if bad:
                 msg2 = ('<br/>' + T('%s articles were malformed')) % len(bad)
             if miss:
                 msg3 = ('<br/>' + T('%s articles were missing')) % len(miss)
             if dups:
                 msg4 = ('<br/>' + T('%s articles had non-matching duplicates')) % len(dups)
-            msg = ''.join((msg1, msg2, msg3, msg4,))
+            if killed:
+                msg5 = ('<br/>' + T('%s articles were removed')) % len(killed)
+            msg = ''.join((msg1, msg2, msg3, msg4, msg5, ))
             self.set_unpack_info('Download', msg, unique=True)
             if self.url:
                 self.set_unpack_info('Source', format_source_url(self.url), unique=True)
@@ -1110,8 +1179,14 @@ class NzbObject(TryList):
                     if article:
                         break
 
+        # Remove all files for which admin could not be read
         for nzf in nzf_remove_list:
+            nzf.deleted = True
+            nzf.completed = True
             self.files.remove(nzf)
+        # If cleanup emptied the active files list, end this job
+        if nzf_remove_list and not self.files:
+            sabnzbd.NzbQueue.do.end_job(self)
 
         if not article:
             # No articles for this server, block for next time
@@ -1228,6 +1303,22 @@ class NzbObject(TryList):
                 except:
                     pass
 
+    def remaining(self):
+        """ Return remaining bytes """
+        bytes_left = 0
+        for nzf in self.files:
+            bytes_left += nzf.bytes_left
+        return bytes_left
+
+    def total_and_remaining(self):
+        """ Return total and remaining bytes """
+        bytes = 0
+        bytes_left = 0
+        for nzf in self.files:
+            bytes += nzf.bytes
+            bytes_left += nzf.bytes_left
+        return bytes, bytes_left
+
     def gather_info(self, for_cli = False):
         bytes_left_all = 0
 
@@ -1360,6 +1451,8 @@ class NzbObject(TryList):
         self.pp_active = False
         self.avg_stamp = time.mktime(self.avg_date.timetuple())
         self.wait = None
+        if self.meta is None:
+            self.meta = {}
         TryList.__init__(self)
 
 
@@ -1370,8 +1463,9 @@ class NzbObject(TryList):
 #-------------------------------------------------------------------------------
 
 def nzf_get_filename(nzf):
-    # Return filename, if the filename not set, try the
-    # the full subject line instead. Can produce non-ideal results
+    """ Return filename, if the filename not set, try the
+        the full subject line instead. Can produce non-ideal results
+    """
     name = nzf.filename
     if not name:
         name = nzf.subject
@@ -1380,12 +1474,35 @@ def nzf_get_filename(nzf):
     return name.lower()
 
 
+def get_ext_list():
+    """ Return priority extenstion list, with extensions starting with a period
+    """
+    exts = []
+    for ext in cfg.prio_sort_list():
+        ext = ext.strip()
+        if not ext.startswith('.'):
+            ext = '.' + ext
+        exts.append(ext)
+    return exts
+
+
+def ext_on_list(name, lst):
+    """ Return True if `name` contains any extension in `lst`
+    """
+    for ext in lst:
+        if name.rfind(ext) >= 0:
+            return True
+    return False
+
+
 def nzf_cmp_date(nzf1, nzf2):
-    # Compare files based on date, but give vol-par files preference
+    """ Compare files based on date, but give vol-par files preference.
+        Wrapper needed, because `cmp` function doesn't handle extra parms.
+    """
     return nzf_cmp_name(nzf1, nzf2, name=False)
 
 
-RE_RAR = re.compile(r'(\.rar|\.r\d\d)|\.s\d\d|\.t\d\d|\.u\d\d|\.v\d\d', re.I)
+RE_RAR = re.compile(r'(\.rar|\.r\d\d|\.s\d\d|\.t\d\d|\.u\d\d|\.v\d\d)$', re.I)
 
 def nzf_cmp_name(nzf1, nzf2, name=True):
     # The comparison will sort .par2 files to the top of the queue followed by .rar files,
@@ -1409,6 +1526,16 @@ def nzf_cmp_name(nzf1, nzf2, name=True):
     if is_par2 and not is_par1:
         return -1
 
+    # Anything with a priority extention goes first
+    ext_list = get_ext_list()
+    if ext_list:
+        onlist1 = ext_on_list(name1, ext_list)
+        onlist2 = ext_on_list(name2, ext_list)
+        if onlist1 and not onlist2:
+            return -1
+        if onlist2 and not onlist1:
+            return 1
+
     if name:
         # Prioritise .rar files above any other type of file (other than vol-par)
         # Useful for nzb streaming
@@ -1507,8 +1634,10 @@ def get_attrib_file(path, size):
         return [None for n in xrange(size)]
 
     for n in xrange(size):
-        line = f.readline().strip('\n ')
+        line = f.readline().strip('\r\n ')
         if line:
+            if line.lower() == 'none':
+                line = None
             try:
                 line = int(line)
             except:
@@ -1532,3 +1661,43 @@ def set_attrib_file(path, attribs):
         f.write('%s\n' % item)
     f.close()
 
+
+def analyse_par2(name):
+    """ Check if file is a par2-file and determine vol/block
+        return head, vol, block
+        head is empty when not a par2 file
+    """
+    head = None
+    vol = block = 0
+    if name and not REJECT_PAR2_RE.search(name):
+        m = PROBABLY_PAR2_RE.search(name)
+        if m:
+            head = m.group(1)
+            vol = m.group(2)
+            block = m.group(3)
+        elif name.lower().find('.par2') > 0:
+            head = os.path.splitext(name)[0].strip()
+        else:
+            head = None
+    return head, vol, block
+
+
+def name_extractor(subject):
+    """ Try to extract a file name from a subject line, return `subject` if in doubt
+    """
+    result = subject
+    for name in re.findall(SUBJECT_FN_MATCHER, subject):
+        name = name.strip(' "')
+        if name and RE_NORMAL_NAME.search(name):
+            result = name
+    return platform_encode(result)
+
+
+def matcher(pattern, txt):
+    """ Return True if `pattern` is sufficiently equal to `txt`
+    """
+    if txt.endswith(pattern):
+        txt = txt[:txt.rfind(pattern)].strip()
+        return (not txt) or txt.endswith('"')
+    else:
+        return False
