@@ -31,12 +31,12 @@ import re
 from sabnzbd.newsunpack import unpack_magic, par2_repair, external_processing, sfv_check
 from threading import Thread
 from sabnzbd.misc import real_path, get_unique_path, create_dirs, move_to_path, \
-                         get_unique_filename, make_script_path, verified_flag_file, \
+                         make_script_path, \
                          on_cleanup_list, renamer, remove_dir, remove_all, globber, \
-                         set_permissions
+                         set_permissions, cleanup_empty_directories
 from sabnzbd.tvsort import Sorter
-from sabnzbd.constants import REPAIR_PRIORITY, POSTPROC_QUEUE_FILE_NAME, \
-     POSTPROC_QUEUE_VERSION, sample_match, JOB_ADMIN, Status
+from sabnzbd.constants import REPAIR_PRIORITY, TOP_PRIORITY, POSTPROC_QUEUE_FILE_NAME, \
+     POSTPROC_QUEUE_VERSION, sample_match, JOB_ADMIN, Status, VERIFIED_FILE
 from sabnzbd.encoding import TRANS, unicoder
 from sabnzbd.newzbin import Bookmarks
 import sabnzbd.emailer as emailer
@@ -184,8 +184,8 @@ class PostProcessor(Thread):
                 sabnzbd.downloader.Downloader.do.wait_for_postproc()
 
             self.__busy = True
-            if process_job(nzo):
-                self.remove(nzo)
+            process_job(nzo)
+            self.remove(nzo)
             check_eoq = True
 
             ## Allow download to proceed
@@ -206,6 +206,8 @@ def process_job(nzo):
     par_error = False
     # keep track of any unpacking errors
     unpack_error = False
+    # Signal empty download, for when 'empty_postproc' is enabled
+    empty = False
     nzb_list = []
     # These need to be initialised incase of a crash
     workdir_complete = ''
@@ -225,19 +227,6 @@ def process_job(nzo):
     filename = nzo.final_name
     msgid = nzo.msgid
 
-    if nzo.precheck:
-        # Check result
-        enough, ratio = nzo.check_quality()
-        if enough:
-            # Enough data present, do real download
-            workdir = nzo.downpath
-            sabnzbd.nzbqueue.NzbQueue.do.cleanup_nzo(nzo, keep_basic=True)
-            sabnzbd.nzbqueue.NzbQueue.do.repair_job(workdir)
-            return True
-        else:
-            # Not enough data, flag as failed
-            nzo.save_attribs()
-
     if cfg.allow_streaming() and not (flag_repair or flag_unpack or flag_delete):
         # After streaming, force +D
         nzo.set_pp(3)
@@ -245,6 +234,12 @@ def process_job(nzo):
         nzo.save_attribs()
         all_ok = False
 
+    if nzo.fail_msg: # Special case: aborted due to too many missing data
+        nzo.status = Status.FAILED
+        nzo.save_attribs()
+        all_ok = False
+        par_error = unpack_error = True
+
     try:
 
         # Get the folder containing the download result
@@ -252,8 +247,9 @@ def process_job(nzo):
         tmp_workdir_complete = None
 
         # if no files are present (except __admin__), fail the job
-        if len(globber(workdir)) < 2:
+        if all_ok and len(globber(workdir)) < 2:
             if nzo.precheck:
+                enough, ratio = nzo.check_quality()
                 req_ratio = float(cfg.req_completion_rate()) / 100.0
                 # Make sure that rounded ratio doesn't equal required ratio
                 # when it is actually below required
@@ -264,13 +260,15 @@ def process_job(nzo):
                 emsg = T('Download might fail, only %s of required %s available') % (emsg, emsg2)
             else:
                 emsg = T('Download failed - Out of your server\'s retention?')
+                empty = True
             nzo.fail_msg = emsg
             nzo.set_unpack_info('Fail', emsg)
             nzo.status = Status.FAILED
             # do not run unpacking or parity verification
             flag_repair = flag_unpack = False
-            par_error = unpack_error = True
-            all_ok = False
+            all_ok = cfg.empty_postproc() and empty
+            if not all_ok:
+                par_error = unpack_error = True
 
         script = nzo.script
         cat = nzo.cat
@@ -280,7 +278,7 @@ def process_job(nzo):
                      filename, flag_repair, flag_unpack, flag_delete, script, cat)
 
         ## Par processing, if enabled
-        if flag_repair:
+        if all_ok and flag_repair:
             par_error, re_add = parring(nzo, workdir)
             if re_add:
                 # Try to get more par files
@@ -311,7 +309,7 @@ def process_job(nzo):
             ## TV/Movie/Date Renaming code part 1 - detect and construct paths
             file_sorter = Sorter(cat)
             complete_dir = file_sorter.detect(dirname, complete_dir)
-            if file_sorter.is_sortfile():
+            if file_sorter.sort_file:
                 one_folder = False
 
             if one_folder:
@@ -357,8 +355,9 @@ def process_job(nzo):
                         for file_ in files:
                             path = os.path.join(root, file_)
                             new_path = path.replace(workdir, tmp_workdir_complete)
-                            new_path = get_unique_filename(new_path)
-                            if not move_to_path(path, new_path, unique=False):
+                            ok, new_path = move_to_path(path, new_path)
+                            newfiles.append(new_path)
+                            if not ok:
                                 nzo.set_unpack_info('Unpack', T('Failed moving %s to %s') % (unicoder(path), unicoder(new_path)))
                                 all_ok = False
                                 break
@@ -382,10 +381,7 @@ def process_job(nzo):
                     nzb_list = None
                 if nzb_list:
                     nzo.set_unpack_info('Download', T('Sent %s to queue') % unicoder(nzb_list))
-                    try:
-                        remove_dir(tmp_workdir_complete)
-                    except:
-                        pass
+                    cleanup_empty_directories(tmp_workdir_complete)
                 else:
                     cleanup_list(tmp_workdir_complete, False)
 
@@ -394,22 +390,28 @@ def process_job(nzo):
         if not nzb_list:
             ## Give destination its final name
             if cfg.folder_rename() and tmp_workdir_complete and not one_folder:
-                if not all_ok:
+                if all_ok:
+                    try:
+                        newfiles = rename_and_collapse_folder(tmp_workdir_complete, workdir_complete, newfiles)
+                    except:
+                        logging.error(Ta('Error renaming "%s" to "%s"'), tmp_workdir_complete, workdir_complete)
+                        logging.info('Traceback: ', exc_info = True)
+                        # Better disable sorting because filenames are all off now
+                        file_sorter.sort_file = None
+                else:
                     workdir_complete = tmp_workdir_complete.replace('_UNPACK_', '_FAILED_')
                     workdir_complete = get_unique_path(workdir_complete, n=0, create_dir=False)
-                try:
-                    collapse_folder(tmp_workdir_complete, workdir_complete)
-                except:
-                    logging.error(Ta('Error renaming "%s" to "%s"'), tmp_workdir_complete, workdir_complete)
-                    logging.info("Traceback: ", exc_info = True)
 
-            job_result = int(par_error) + int(unpack_error)*2
+            if empty:
+                job_result = -1
+            else:
+                job_result = int(par_error) + int(unpack_error)*2
 
             if cfg.ignore_samples() > 0:
                 remove_samples(workdir_complete)
 
             ## TV/Movie/Date Renaming code part 2 - rename and move files to parent folder
-            if all_ok and file_sorter.is_sortfile():
+            if all_ok and file_sorter.sort_file:
                 if newfiles:
                     file_sorter.rename(newfiles, workdir_complete)
                     workdir_complete, ok = file_sorter.move(workdir_complete)
@@ -421,7 +423,7 @@ def process_job(nzo):
 
             ## Run the user script
             script_path = make_script_path(script)
-            if all_ok and (not nzb_list) and script_path:
+            if (all_ok or not cfg.safe_postproc()) and (not nzb_list) and script_path:
                 #set the current nzo status to "Ext Script...". Used in History
                 nzo.status = Status.RUNNING
                 nzo.set_action_line(T('Running script'), unicoder(script))
@@ -444,7 +446,7 @@ def process_job(nzo):
         if (not nzb_list) and cfg.email_endjob():
             if (cfg.email_endjob() == 1) or (cfg.email_endjob() == 2 and (unpack_error or par_error)):
                 emailer.endjob(dirname, msgid, cat, all_ok, workdir_complete, nzo.bytes_downloaded,
-                               nzo.unpack_info, script, TRANS(script_log), script_ret)
+                               nzo.fail_msg, nzo.unpack_info, script, TRANS(script_log), script_ret)
 
         if script_output:
             # Can do this only now, otherwise it would show up in the email
@@ -471,6 +473,9 @@ def process_job(nzo):
         elif all_ok and isinstance(nzo.url, str):
             sabnzbd.proxy_rm_bookmark(nzo.url)
 
+        ## Force error for empty result
+        all_ok = all_ok and not empty
+
         ## Show final status in history
         if all_ok:
             growler.send_notification(T('Download Completed'), filename, 'complete')
@@ -489,11 +494,9 @@ def process_job(nzo):
         nzo.status = Status.FAILED
         par_error = True
         all_ok = False
-        info = nzo.unpack_info.copy()
-        info['fail'] = [nzo.fail_msg]
         if cfg.email_endjob():
             emailer.endjob(dirname, msgid, cat, all_ok, workdir_complete, nzo.bytes_downloaded,
-                           info, '', '', 0)
+                           nzo.fail_msg, nzo.unpack_info, '', '', 0)
 
 
     if all_ok:
@@ -539,10 +542,14 @@ def process_job(nzo):
 def parring(nzo, workdir):
     """ Perform par processing. Returns: (par_error, re_add)
     """
+    assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject)
     filename = nzo.final_name
     growler.send_notification(T('Post-processing'), nzo.final_name, 'pp')
     logging.info('Par2 check starting on %s', filename)
 
+    ## Get verification status of sets
+    verified = sabnzbd.load_data(VERIFIED_FILE, nzo.workpath, remove=False) or {}
+
     ## Collect the par files
     if nzo.partable:
         par_table = nzo.partable.copy()
@@ -554,48 +561,68 @@ def parring(nzo, workdir):
     par_error = False
 
     if repair_sets:
-
-        for set_ in repair_sets:
-            logging.info("Running repair on set %s", set_)
-            parfile_nzf = par_table[set_]
-            need_re_add, res = par2_repair(parfile_nzf, nzo, workdir, set_)
-            if need_re_add:
-                re_add = True
-            else:
+        for setname in repair_sets:
+            if cfg.ignore_samples() > 0 and 'sample' in setname.lower():
+                continue
+            if not verified.get(setname, False):
+                logging.info("Running repair on set %s", setname)
+                parfile_nzf = par_table[setname]
+                if not os.path.exists(os.path.join(nzo.downpath, parfile_nzf.filename)):
+                    continue
+                need_re_add, res = par2_repair(parfile_nzf, nzo, workdir, setname)
+                re_add = re_add or need_re_add
+                if not res and not need_re_add and cfg.sfv_check():
+                    res = try_sfv_check(nzo, workdir, setname)
+                verified[setname] = res
                 par_error = par_error or not res
-
-        if re_add:
-            logging.info('Readded %s to queue', filename)
-            nzo.priority = REPAIR_PRIORITY
-            sabnzbd.nzbqueue.add_nzo(nzo)
-            sabnzbd.downloader.Downloader.do.resume_from_postproc()
-
-        logging.info('Par2 check finished on %s', filename)
-
-    if (par_error and not re_add) or not repair_sets:
-        # See if alternative SFV check is possible
+    else:
+        logging.info("No par2 sets for %s", filename)
+        nzo.set_unpack_info('Repair', T('[%s] No par2 sets') % unicoder(filename))
         if cfg.sfv_check():
-            sfvs = globber(workdir, '*.sfv')
-        else:
-            sfvs = None
-        if sfvs:
-            par_error = False
-            nzo.set_unpack_info('Repair', T('Trying SFV verification'))
-            for sfv in sfvs:
-                if not sfv_check(sfv):
-                    nzo.set_unpack_info('Repair', T('Some files failed to verify against "%s"') % unicoder(os.path.basename(sfv)))
-                    par_error = True
-            if not par_error:
-                nzo.set_unpack_info('Repair', T('Verified successfully using SFV files'))
-        elif not repair_sets:
-            logging.info("No par2 sets for %s", filename)
-            nzo.set_unpack_info('Repair', T('[%s] No par2 sets') % unicoder(filename))
+            par_error = not try_sfv_check(nzo, workdir, '')
+            verified[''] = not par_error
 
-    if not par_error:
-        verified_flag_file(workdir, create=True)
+    if re_add:
+        logging.info('Readded %s to queue', filename)
+        if nzo.priority != TOP_PRIORITY:
+            nzo.priority = REPAIR_PRIORITY
+        sabnzbd.nzbqueue.add_nzo(nzo)
+        sabnzbd.downloader.Downloader.do.resume_from_postproc()
+
+    sabnzbd.save_data(verified, VERIFIED_FILE, nzo.workpath)
+
+    logging.info('Par2 check finished on %s', filename)
     return par_error, re_add
 
 
+def try_sfv_check(nzo, workdir, setname):
+    """ Attempt to verify set using SFV file
+        Return True if verified, False when failed
+        When setname is '', all SFV files will be used, otherwise only the matching one
+        When setname is '' and no SFV files are found, True is returned
+        """
+    # Get list of SFV names; shortest name first, minimizes the chance on a mismatch
+    sfvs = globber(workdir, '*.sfv')
+    sfvs.sort(lambda x, y: len(x) - len(y))
+    par_error = False
+    found = False
+    for sfv in sfvs:
+        if setname in os.path.basename(sfv):
+            found = True
+            nzo.set_unpack_info('Repair', T('Trying SFV verification'))
+            failed = sfv_check(sfv)
+            if failed:
+                msg = T('Some files failed to verify against "%s"') % unicoder(os.path.basename(sfv))
+                msg += '; '
+                msg += '; '.join(failed)
+                nzo.set_unpack_info('Repair', msg)
+                par_error = True
+            else:
+                nzo.set_unpack_info('Repair', T('Verified successfully using SFV files'))
+            if setname:
+                break
+    return (found or not setname) and not par_error
+
 
 #------------------------------------------------------------------------------
 
@@ -648,6 +675,11 @@ def cleanup_list(wdir, skip_nzb):
                     except:
                         logging.error(Ta('Removing %s failed'), path)
                         logging.info("Traceback: ", exc_info = True)
+        if files:
+            try:
+                remove_dir(wdir)
+            except:
+                pass
 
 
 def prefix(path, pre):
@@ -663,29 +695,24 @@ def nzb_redirect(wdir, nzbname, pp, script, cat, priority):
         if so send to queue and remove if on CleanList
         Returns list of processed NZB's
     """
-    lst = []
-
-    try:
-        files = os.listdir(wdir)
-    except:
-        files = []
+    files = []
+    for root, dirs, names in os.walk(wdir):
+        for name in names:
+            files.append(os.path.join(root, name))
 
     for file_ in files:
         if os.path.splitext(file_)[1].lower() != '.nzb':
-            return lst
+            return None
 
-    # For a single NZB, use the current job name
+    # For multiple NZBs, cannot use the current job name
     if len(files) != 1:
         nzbname = None
 
     # Process all NZB files
     for file_ in files:
-        if file_.lower().endswith('.nzb'):
-            dirscanner.ProcessSingleFile(file_, os.path.join(wdir, file_), pp, script, cat,
-                                         priority=priority, keep=False, dup_check=False, nzbname=nzbname)
-            lst.append(file_)
-
-    return lst
+        dirscanner.ProcessSingleFile(os.path.split(file_)[1], file_, pp, script, cat,
+                                     priority=priority, keep=False, dup_check=False, nzbname=nzbname)
+    return files
 
 
 def one_file_or_folder(folder):
@@ -726,9 +753,10 @@ def remove_samples(path):
 
 
 #------------------------------------------------------------------------------
-def collapse_folder(oldpath, newpath):
+def rename_and_collapse_folder(oldpath, newpath, files):
     """ Rename folder, collapsing when there's just a single subfolder
         oldpath --> newpath OR oldpath/subfolder --> newpath
+        Modify list of filenames accordingly
     """
     orgpath = oldpath
     items = globber(oldpath)
@@ -739,11 +767,16 @@ def collapse_folder(oldpath, newpath):
             logging.info('Collapsing %s', os.path.join(newpath, folder))
             oldpath = folder_path
 
+    oldpath = os.path.normpath(oldpath)
+    newpath = os.path.normpath(newpath)
+    files = [os.path.normpath(f).replace(oldpath, newpath) for f in files]
+
     renamer(oldpath, newpath)
     try:
         remove_dir(orgpath)
     except:
         pass
+    return files
 
 
 #------------------------------------------------------------------------------
