@@ -49,7 +49,6 @@ class URLGrabber(Thread):
 
     def __init__(self):
         Thread.__init__(self)
-        now = time.time()
         self.queue = Queue.Queue()
         for tup in NzbQueue.do.get_urls():
             url, nzo = tup
@@ -115,7 +114,10 @@ class URLGrabber(Thread):
                     logging.info('Removing nzbmatrix bookmark %s', matrix_id)
                 else:
                     logging.info('Grabbing URL %s', url)
-                opener = urllib.FancyURLopener({})
+                if '.nzbsrus.' in url:
+                    opener = urllib.URLopener({})
+                else:
+                    opener = urllib.FancyURLopener({})
                 opener.prompt_user_passwd = None
                 opener.addheaders = []
                 opener.addheader('User-Agent', 'SABnzbd+/%s' % sabnzbd.version.__version__)
@@ -164,20 +166,21 @@ class URLGrabber(Thread):
                             logging.error(msg)
                             misc.bad_fetch(future_nzo, clean_matrix_url(url), msg, retry=True)
                         continue
-                    category = _MATRIX_MAP.get(category, category)
+                    category = get_matrix_category(url, category)
+
+                    if del_bookmark:
+                        # No retries of nzbmatrix bookmark removals
+                        continue
+
                 else:
-                    msg = ''
-                    retry = True
-
-                if del_bookmark:
-                    # No retries of nzbmatrix bookmark removals
-                    continue
-
-                # Check if the filepath is specified, if not, check if a retry is allowed.
-                if not fn:
-                    logging.info('Retry URL %s', url)
-                    self.add(url, future_nzo, 5)
-                    continue
+                    fn, msg, retry, wait = _analyse_others(fn, url)
+                    if not fn:
+                        if retry:
+                            logging.info('Retry URL %s', url)
+                            self.add(url, future_nzo, wait)
+                        else:
+                            misc.bad_fetch(future_nzo, url, msg, retry=True)
+                        continue
 
                 if not filename:
                     filename = os.path.basename(url) + '.nzb'
@@ -204,6 +207,10 @@ class URLGrabber(Thread):
                         if res == -2:
                             logging.info('Incomplete NZB, retry after 5 min %s', url)
                             when = 300
+                        elif res == -1:
+                            # Error, but no reason to retry. Warning is already given
+                            NzbQueue.do.remove(future_nzo.nzo_id, add_to_history=False)
+                            continue
                         else:
                             logging.info('Unknown error fetching NZB, retry after 2 min %s', url)
                             when = 120
@@ -228,7 +235,8 @@ class URLGrabber(Thread):
 
 
 #-------------------------------------------------------------------------------
-_RE_NZBMATRIX = re.compile(r'nzbmatrix.com/(.*)[\?&]id=(\d+)', re.I)
+_RE_NZBMATRIX = re.compile(r'nzbmatrix\.com/(.*)[\?&]id=(\d+)', re.I)
+_RE_NZBXXX    = re.compile(r'nzbxxx\.com/(.*)[\?&]id=(\d+)', re.I)
 _RE_NZBMATRIX_USER = re.compile(r'&username=([^&=]+)', re.I)
 _RE_NZBMATRIX_API  = re.compile(r'&apikey=([^&=]+)', re.I)
 
@@ -237,22 +245,38 @@ def _matrix_url(url):
 
     matrix_id = 0
     m = _RE_NZBMATRIX.search(url)
+    if not m:
+        mx = _RE_NZBXXX.search(url)
+
+    if m:
+        site = 'nzbmatrix.com'
+        user = urllib.quote_plus(cfg.matrix_username())
+        key = urllib.quote_plus(cfg.matrix_apikey())
+    elif mx:
+        site = 'nzbxxx.com'
+        user = urllib.quote_plus(cfg.xxx_username())
+        key = urllib.quote_plus(cfg.xxx_apikey())
+        m = mx
+
     if m:
         matrix_id = m.group(2)
         if not _RE_NZBMATRIX_USER.search(url) or not _RE_NZBMATRIX_API.search(url):
-            user = urllib.quote_plus(cfg.matrix_username())
-            key = urllib.quote_plus(cfg.matrix_apikey())
-            url = '%s://api.nzbmatrix.com/v1.1/download.php?id=%s&username=%s&apikey=%s' % \
-                  (_PROTOCOL, matrix_id, user, key)
+            url = '%s://api.%s/v1.1/download.php?id=%s&username=%s&apikey=%s' % \
+                  (_PROTOCOL, site, matrix_id, user, key)
     return url, matrix_id
 
 
 def clean_matrix_url(url):
     ''' Return nzbmatrix url without user credentials '''
+    site = 'nzbmatrix.com'
     m = _RE_NZBMATRIX.search(url)
+    if not m:
+        m = _RE_NZBXXX.search(url)
+        site = 'nzbxxx.com'
+
     if m:
         matrix_id = m.group(2)
-        url = '%s://api.nzbmatrix.com/v1.1/download.php?id=%s' % (_PROTOCOL, matrix_id)
+        url = '%s://api.%s/v1.1/download.php?id=%s' % (_PROTOCOL, site, matrix_id)
     return url
 
 
@@ -313,6 +337,50 @@ def _analyse_matrix(fn, matrix_id):
     return fn, msg, False, 0
 
 
+
+RUS_FATAL = ('DENIED_MISSING_CREDENTIALS', 'DENIED_NO_ACCOUNT',
+             'DENIED_INVALID_CREDENTIALS', 'INCORRECT_URL',
+             'NZB_DELETED', 'POST_NUKED', 'FILE_UNAVAILABLE'
+            )
+RUS_15M =   ('SQL_ERROR', 'SERVICE_OFFLINE')
+RUS_60M =   ('MAX_DOWNLOAD_REACHED_UPGRADE_TO_VIP', 'MAX_DOWNLOAD_REACHED')
+
+def _analyse_others(fn, url):
+    """ Analyse respons of indexer
+        returns fn|None, error-message|None, retry, wait-seconds
+    """
+    msg = ''
+    wait = 0
+    if not fn:
+        logging.debug('No response from indexer, retry after 60 sec')
+        return None, msg, True, 60
+    try:
+        f = open(fn, 'r')
+        data = f.read(100)
+        f.close()
+    except:
+        logging.debug('Problem with tempfile %s from indexer, retry after 60 sec', fn)
+        return None, msg, True, 60
+
+    # Check for an error response
+    if not data:
+        logging.debug('Received nothing from indexer, retry after 60 sec')
+        return None, msg, True, 60
+
+    if '.nzbsrus.' in url:
+        # Partial support for nzbsrus.com's API
+        if misc.match_str(data, RUS_FATAL):
+            logging.debug('nzbsrus says: %s, abort', data)
+            return None, data, False, 0
+        if misc.match_str(data, RUS_15M):
+            logging.debug('nzbsrus says: %s, wait 15m', data)
+            return None, data, True, 900
+        if misc.match_str(data, RUS_60M):
+            logging.debug('nzbsrus says: %s, wait 60m', data)
+            return None, data, True, 3600
+
+    return fn, msg, False, 0
+
 #------------------------------------------------------------------------------
 _MATRIX_MAP = {
 '28' : 'anime.all',
@@ -334,13 +402,13 @@ _MATRIX_MAP = {
 '13' : 'games.xbox',
 '14' : 'games.xbox360',
 '56' : 'games.xbox360 (other)',
-'54' : 'movies.brrip',
-'2'  : 'movies.divx/xvid',
-'1'  : 'movies.dvd',
-'50' : 'movies.hd (image)',
+'1'  : 'movies.sd (image)',
+'2'  : 'movies.sd',
+'54' : 'movies.hd (remux)',
 '42' : 'movies.hd (x264)',
+'50' : 'movies.hd (image)',
 '4'  : 'movies.other',
-'24' : 'music.dvd',
+'24' : 'music.sd (image)',
 '23' : 'music.lossless',
 '22' : 'music.mp3, albums',
 '47' : 'music.mp3, singles',
@@ -354,7 +422,7 @@ _MATRIX_MAP = {
 '38' : 'other.iOS/iPhone',
 '40' : 'other.other',
 '26' : 'other.radio',
-'5'  : 'tv.dvd (image)',
+'5'  : 'tv.sd (image)',
 '57' : 'tv.hd (image)',
 '41' : 'tv.hd (x264)',
 '8'  : 'tv.other',
@@ -362,3 +430,9 @@ _MATRIX_MAP = {
 '7'  : 'tv.sport/ent'
 }
 
+def get_matrix_category(url, category):
+    category = _MATRIX_MAP.get(category, category)
+    if category and 'nzbxxx.com' in url:
+        return 'XXX: ' + category
+    else:
+        return category
